@@ -107,25 +107,47 @@ typedef void (*sig_t_)(int);
             #define AVOID_BLAS_SYR
         #endif
     #endif
+    #include <stdarg.h>
+    #include <stdio.h>
+    #ifdef _WIN32
+        #define IMPORTED_FUN __declspec(dllimport)
+    #else
+        #define IMPORTED_FUN 
+    #endif
+    extern IMPORTED_FUN void PySys_WriteStdout(const char *fmt, ...);
+    extern IMPORTED_FUN void PySys_WriteStderr(const char *fmt, ...);
+    void python_printmsg(char *msg);
+    void python_printerrmsg(char *msg);
+    void py_printf(const char *fmt, ...);
+    void py_errprintf(void *ignored, const char *fmt, ...);
+    extern void cy_printf(char *msg);
+    extern void cy_errprintf(char *msg);
+    #define printf py_printf
+    #define fprintf py_errprintf
+    #define fflush(arg) {}
 #elif defined(_FOR_R)
     #include <R.h>
     #include <Rinternals.h>
     #include <R_ext/Print.h>
     #include <R_ext/BLAS.h>
     #include <R_ext/Lapack.h>
+    #include <R_ext/Visibility.h>
     #define USE_DOUBLE
-    #define FORCE_NO_NAN_PROPAGATION
     #define printf Rprintf
     #define fprintf(f, message) REprintf(message)
+    #define fflush(f) R_FlushConsole()
 #elif defined(MKL_ILP64)
     #include "mkl.h"
 #endif
 /* Here one may also include the standard headers "cblas.h" and "lapack.h",
    if one wants to use a non-standard version such as ILP64 (-DMKL_ILP64). */
+#if !defined(_FOR_R) && !defined(_FOR_PYTHON)
+    #include <stdio.h>
+#endif
 
 /* Aliasing for compiler optimizations */
 #ifdef __cplusplus
-    #if defined(__GNUG__) || defined(__GNUC__) || defined(_MSC_VER) || defined(__clang__) || defined(__INTEL_COMPILER)
+    #if defined(__GNUG__) || defined(__GNUC__) || defined(_MSC_VER) || defined(__clang__) || defined(__INTEL_COMPILER) || defined(__IBMCPP__) || defined(__ibmxl__)
         #define restrict __restrict
     #else
         #define restrict 
@@ -183,6 +205,7 @@ typedef void (*sig_t_)(int);
     #define fabs_t fabs
     #define fmax_t fmax
     #define sqrt_t sqrt
+    #define fma_t fma
     #define EPSILON_T DBL_EPSILON
     #define HUGE_VAL_T HUGE_VAL
     #define cblas_tdot cblas_ddot
@@ -209,6 +232,7 @@ typedef void (*sig_t_)(int);
     #define fmax_t fmaxf
     #define fabs_t fabsf
     #define sqrt_t sqrtf
+    #define fma_t fmaf
     #define EPSILON_T FLT_EPSILON
     #define HUGE_VAL_T HUGE_VALF
     #define cblas_tdot cblas_sdot
