@@ -127,6 +127,8 @@ void copy_arr_(real_t *restrict src, real_t *restrict dest, size_t n, int nthrea
     }
 }
 
+/* Note: the C99 standard only guarantes that isnan(NAN)!=0, and some compilers
+   like mingw64 will NOT make isnan(NAN)==1. */
 int_t count_NAs(real_t arr[], size_t n, int nthreads)
 {
     int_t cnt_NA = 0;
@@ -141,7 +143,7 @@ int_t count_NAs(real_t arr[], size_t n, int nthreads)
 
     #pragma omp parallel for schedule(static) num_threads(nthreads) shared(arr, n) reduction(+:cnt_NA)
     for (size_t_for ix = 0; ix < n; ix++)
-        cnt_NA += isnan(arr[ix]);
+        cnt_NA += isnan(arr[ix]) != 0;
     if (cnt_NA < 0) cnt_NA = INT_MAX; /* <- overflow */
     return cnt_NA;
 }
@@ -166,7 +168,7 @@ void count_NAs_by_row
     {
         int_t cnt = 0;
         for (size_t col = 0; col < (size_t)n; col++)
-            cnt += isnan(arr[col + row*n]);
+            cnt += isnan(arr[col + row*n]) != 0;
         cnt_NA[row] = cnt;
     }
 
@@ -215,7 +217,7 @@ void count_NAs_by_col
 {
     for (size_t row = 0; row < (size_t)m; row++)
         for (size_t col = 0; col < (size_t)n; col++)
-            cnt_NA[col] += isnan(arr[col + row*n]);
+            cnt_NA[col] += isnan(arr[col + row*n]) != 0;
 
     *full_dense = true;
     for (int_t ix = 0; ix < n; ix++) {
@@ -235,7 +237,7 @@ void count_NAs_by_col
             *near_dense = true;
     }
 
-    *some_full = full_dense;
+    *some_full = *full_dense;
     if (!(*full_dense))
     {
         for (int_t ix = 0; ix < n; ix++)
@@ -476,7 +478,7 @@ void taxpy_large(real_t *restrict A, real_t x, real_t *restrict Y, size_t n, int
         else
             #pragma omp parallel for schedule(static) num_threads(nthreads) shared(A, x, Y, n)
             for (size_t_for ix = 0; ix < n; ix++)
-                Y[ix] += x*A[ix];
+                Y[ix] = fma_t(x, A[ix], Y[ix]);
     }
 }
 
@@ -622,7 +624,6 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
     #ifndef USE_XOSHIRO128
     const uint64_t two53_i = (UINT64_C(1) << 53) - UINT64_C(1);
     #endif
-    const double two53_d = (double)(UINT64_C(1) << 53);
     const double twoPI = 2. * M_PI;
     uint64_t rnd1, rnd2;
     #ifdef USE_XOSHIRO128
@@ -647,7 +648,7 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
             rnd2 = xoshiro256pp(state);
             #endif
 
-            #if defined(DBL_MANT_DIG) && (DBL_MANT_DIG == 53)
+            #if defined(DBL_MANT_DIG) && (DBL_MANT_DIG == 53) &&(FLT_RADIX == 2)
             #ifdef USE_XOSHIRO128
             if (is_little_endian) {
                 rnd12 = rnd12 & two21_i;
@@ -660,11 +661,11 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
             memcpy((char*)&rnd1 + sizeof(uint32_t), &rnd12, sizeof(uint32_t));
             memcpy((char*)&rnd2, &rnd21, sizeof(uint32_t));
             memcpy((char*)&rnd2 + sizeof(uint32_t), &rnd22, sizeof(uint32_t));
-            u = (double)rnd1 / two53_d;
-            v = (double)rnd2 / two53_d;
+            u = ldexp((double)rnd1, -53);
+            v = ldexp((double)rnd2, -53);
             #else
-            u = (double)(rnd1 & two53_i) / two53_d;
-            v = (double)(rnd2 & two53_i) / two53_d;
+            u = ldexp((double)(rnd1 & two53_i), -53);
+            v = ldexp((double)(rnd2 & two53_i), -53);
             #endif
             #else
             u = (double)rnd1 / (double)UINT64_MAX;
@@ -674,8 +675,8 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
         while (u == 0 || v == 0);
 
         u = sqrt(-2. * log(u));
-        seq[(size_t)2*ix] = (real_t)((cos(twoPI * v) * u) / 128.);
-        seq[(size_t)2*ix + (size_t)1] = (real_t)((sin(twoPI * v) * u) / 128.);
+        seq[(size_t)2*ix] = (real_t)ldexp(cos(twoPI * v) * u, -7);
+        seq[(size_t)2*ix + (size_t)1] = (real_t)ldexp(sin(twoPI * v) * u, -7);
     }
 
     if ((n % (size_t)2) != 0)
@@ -692,7 +693,7 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
             rnd2 = xoshiro256pp(state);
             #endif
 
-            #if defined(DBL_MANT_DIG) && (DBL_MANT_DIG == 53)
+            #if defined(DBL_MANT_DIG) && (DBL_MANT_DIG == 53) &&(FLT_RADIX == 2)
             #ifdef USE_XOSHIRO128
             if (is_little_endian) {
                 rnd12 = rnd12 & two21_i;
@@ -705,11 +706,11 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
             memcpy((char*)&rnd1 + sizeof(uint32_t), &rnd12, sizeof(uint32_t));
             memcpy((char*)&rnd2, &rnd21, sizeof(uint32_t));
             memcpy((char*)&rnd2 + sizeof(uint32_t), &rnd22, sizeof(uint32_t));
-            u = (double)rnd1 / two53_d;
-            v = (double)rnd2 / two53_d;
+            u = ldexp((double)rnd1, -53);
+            v = ldexp((double)rnd2, -53);
             #else
-            u = (double)(rnd1 & two53_i) / two53_d;
-            v = (double)(rnd2 & two53_i) / two53_d;
+            u = ldexp((double)(rnd1 & two53_i), -53);
+            v = ldexp((double)(rnd2 & two53_i), -53);
             #endif
             #else
             u = (double)rnd1 / (double)UINT64_MAX;
@@ -719,7 +720,7 @@ void rnorm_xoshiro(real_t *seq, const size_t n, rng_state_t state[4])
         while (u == 0 || v == 0);
 
         u = sqrt(-2. * log(u));
-        seq[n - (size_t)1] = (real_t)((cos(twoPI * v) * u) / 128.);
+        seq[n - (size_t)1] = (real_t)ldexp(cos(twoPI * v) * u, -7);
     }
 }
 #else
@@ -727,7 +728,6 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
 {
     const uint32_t two25_i = (UINT32_C(1) << 25) - UINT32_C(1);
     const int32_t two24_i = (UINT32_C(1) << 24);
-    const float two24_f = (float)(UINT32_C(1) << 24);
     uint32_t rnd1, rnd2;
     #ifndef USE_XOSHIRO128
     uint64_t rnd0;
@@ -747,9 +747,9 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
             memcpy(&rnd2, (char*)&rnd0 + sizeof(uint32_t), sizeof(uint32_t));
             #endif
 
-            #if defined(FLT_MANT_DIG) && (FLT_MANT_DIG == 24)
-            u = (float)((int32_t)(rnd1 & two25_i) - two24_i) / two24_f;
-            v = (float)((int32_t)(rnd2 & two25_i) - two24_i) / two24_f;
+            #if defined(FLT_MANT_DIG) && (FLT_MANT_DIG == 24) &&(FLT_RADIX == 2)
+            u = ldexpf((float)((int32_t)(rnd1 & two25_i) - two24_i), -24);
+            v = ldexpf((float)((int32_t)(rnd2 & two25_i) - two24_i), -24);
             #else
             u = (float)rnd1 / (float)INT32_MAX;
             v = (float)rnd2 / (float)INT32_MAX;
@@ -760,8 +760,8 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
         while (s == 0 || s >= 1);
 
         s = sqrtf((-2.0f / s) * logf(s));
-        seq[(size_t)2*ix] = (real_t)((u * s) / 128.0f);
-        seq[(size_t)2*ix + (size_t)1] = (real_t)((v * s) / 128.0f);
+        seq[(size_t)2*ix] = ldexpf(u * s, -7);
+        seq[(size_t)2*ix + (size_t)1] = ldexpf(v * s, -7);
     }
 
     if ((n % (size_t)2) != 0)
@@ -777,9 +777,9 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
             memcpy(&rnd2, (char*)&rnd0 + sizeof(uint32_t), sizeof(uint32_t));
             #endif
 
-            #if defined(FLT_MANT_DIG) && (FLT_MANT_DIG == 24)
-            u = (float)((int32_t)(rnd1 & two25_i) - two24_i) / two24_f;
-            v = (float)((int32_t)(rnd2 & two25_i) - two24_i) / two24_f;
+            #if defined(FLT_MANT_DIG) && (FLT_MANT_DIG == 24) &&(FLT_RADIX == 2)
+            u = ldexpf((float)((int32_t)(rnd1 & two25_i) - two24_i), -24);
+            v = ldexpf((float)((int32_t)(rnd2 & two25_i) - two24_i), -24);
             #else
             u = (float)rnd1 / (float)INT32_MAX;
             v = (float)rnd2 / (float)INT32_MAX;
@@ -790,7 +790,7 @@ void rnorm_xoshiro(float *seq, const size_t n, rng_state_t state[4])
         while (s == 0 || s >= 1);
 
         s = sqrtf((-2.0f / s) * logf(s));
-        seq[n - (size_t)1] = (real_t)((u * s) / 128.0f);
+        seq[n - (size_t)1] = ldexpf(u * s, -7);
     }
 }
 #endif
@@ -842,9 +842,9 @@ int_t rnorm_parallel(ArraysToFill arrays, int_t seed, int nthreads)
     #ifdef USE_R_RNG
     GetRNGstate();
     for (size_t ix = 0; ix < arrays.sizeA; ix++)
-        A[ix] = norm_rand();
+        arrays.A[ix] = norm_rand();
     for (size_t ix = 0; ix < arrays.sizeB; ix++)
-        B[ix] = norm_rand();
+        arrays.B[ix] = norm_rand();
     PutRNGstate();
     return 0;
     #endif
@@ -1399,9 +1399,7 @@ int_t lbfgs_printer_collective
     if ((k % print_every) == 0 && print_every > 0) {
         printf("Iteration %-4d - f(x)= %-8.03g - ||g(x)||= %-8.03g - ls=% 2d\n",
                k, fx, gnorm, ls);
-        #if !defined(_FOR_R)
         fflush(stdout);
-        #endif
     }
     if (should_stop_procedure)
         return 1;
@@ -1427,9 +1425,7 @@ int_t lbfgs_printer_offsets
     if ((k % print_every) == 0 && print_every > 0) {
         printf("Iteration %-5d - f(x)= %-8.03g - ||g(x)||= %-8.03g - ls=% 2d\n",
                k, fx, gnorm, ls);
-        #if !defined(_FOR_R)
         fflush(stdout);
-        #endif
     }
     if (should_stop_procedure)
         return 1;
@@ -1514,10 +1510,12 @@ void fill_lower_triangle(real_t A[], size_t n, size_t lda)
 
 void print_err_msg(const char *msg)
 {
-    fprintf(stderr, msg);
     #ifndef _FOR_R
-    fflush(stderr);
+    fprintf(stderr, "%s", msg);
+    #else
+    fprintf(stderr, msg);
     #endif
+    fflush(stderr);
 }
 
 void print_oom_message(void)
@@ -1571,7 +1569,7 @@ long double compensated_sum_product(real_t *restrict arr1, real_t *restrict arr2
 
     for (size_t ix = 0; ix < n; ix++)
     {
-        diff = arr1[ix]*arr2[ix] - err;
+        diff = fmal(arr1[ix], arr2[ix], -err);
         temp = res + diff;
         err = (temp - res) - diff;
         res = temp;
@@ -1590,7 +1588,7 @@ void custom_syr(const int_t n, const real_t alpha, const real_t *restrict x, rea
         temp = alpha*x[i];
         Arow = A + (size_t)i*(size_t)lda;
         for (int j = i; j < n; j++)
-            Arow[j] += temp*x[j];
+            Arow[j] = fma_t(temp, x[j], Arow[j]);
     }
 }
 #endif
